@@ -1,5 +1,5 @@
 # rawdog: RSS aggregator without delusions of grandeur.
-# Copyright 2003, 2004, 2005 Adam Sampson <ats@offog.org>
+# Copyright 2003, 2004, 2005, 2006 Adam Sampson <ats@offog.org>
 #
 # rawdog is free software; you can redistribute and/or modify it
 # under the terms of that license as published by the Free Software
@@ -16,7 +16,7 @@
 # Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301, USA, or see http://www.gnu.org/.
 
-VERSION = "2.9"
+VERSION = "2.11"
 STATE_VERSION = 2
 import feedparser, feedfinder, plugins
 from persister import Persistable, Persister
@@ -40,13 +40,27 @@ def set_socket_timeout(n):
 		import timeoutsocket
 		timeoutsocket.setDefaultSocketTimeout(n)
 
+def get_system_encoding():
+	"""Get the system encoding."""
+	try:
+		# This doesn't exist on Python 2.2.
+		return locale.getpreferredencoding()
+	except:
+		return "UTF-8"
+
+def safe_ftime(format, t):
+	"""Format a time value into a string in the current locale (as
+	time.strftime), but encode the result as ASCII HTML."""
+	u = unicode(time.strftime(format, t), get_system_encoding())
+	return encode_references(u)
+
 def format_time(secs, config):
 	"""Format a time and date nicely."""
 	t = time.localtime(secs)
 	format = config["datetimeformat"]
 	if format is None:
 		format = config["timeformat"] + ", " + config["dayformat"]
-	return time.strftime(format, t)
+	return safe_ftime(format, t)
 
 def encode_references(s):
 	"""Encode characters in a Unicode string using HTML references."""
@@ -72,12 +86,13 @@ def sanitise_html(html, baseurl, inline, config):
 		return None
 
 	html = encode_references(html)
+	type = "text/html"
 
 	# sgmllib handles "<br/>/" as a SHORTTAG; this workaround from
 	# feedparser.
 	html = re.sub(r'(\S)/>', r'\1 />', html)
-	html = feedparser._resolveRelativeURIs(html, baseurl, "UTF-8")
-	p = feedparser._HTMLSanitizer("UTF-8")
+	html = feedparser._resolveRelativeURIs(html, baseurl, "UTF-8", type)
+	p = feedparser._HTMLSanitizer("UTF-8", type)
 	p.feed(html)
 	html = p.output()
 
@@ -118,7 +133,9 @@ def select_detail(details):
 
 	ds = []
 	for detail in details:
-		ctype = detail["type"]
+		ctype = detail.get("type", None)
+		if ctype is None:
+			continue
 		if types.has_key(ctype):
 			score = types[ctype]
 		else:
@@ -162,11 +179,11 @@ def author_to_html(entry, feedurl, config):
 	if author_detail is not None:
 		if author_detail.has_key("url"):
 			url = author_detail["url"]
-		elif author_detail.has_key("email"):
+		elif author_detail.has_key("email") and author_detail["email"] is not None:
 			url = "mailto:" + author_detail["email"]
-		if author_detail.has_key("email"):
+		if author_detail.has_key("email") and author_detail["email"] is not None:
 			fallback = author_detail["email"]
-		elif author_detail.has_key("url"):
+		elif author_detail.has_key("url") and author_detail["url"] is not None:
 			fallback = author_detail["url"]
 
 	if name == "":
@@ -195,6 +212,8 @@ def fill_template(template, bits):
 	if result.value is not None:
 		return result.value
 
+	encoding = get_system_encoding()
+
 	f = StringIO()
 	if_stack = []
 	def write(s):
@@ -214,7 +233,7 @@ def fill_template(template, bits):
 					if_stack.append(not if_stack.pop())
 			elif bits.has_key(key):
 				if type(bits[key]) == types.UnicodeType:
-					write(bits[key].encode(locale.getpreferredencoding()))
+					write(bits[key].encode(encoding))
 				else:
 					write(bits[key])
 		else:
@@ -510,13 +529,13 @@ class DayWriter:
 
 	def start_day(self, tm):
 		print >>self.file, '<div class="day">'
-		day = time.strftime(self.config["dayformat"], tm)
+		day = safe_ftime(self.config["dayformat"], tm)
 		print >>self.file, '<h2>' + day + '</h2>'
 		self.counter += 1
 
 	def start_time(self, tm):
 		print >>self.file, '<div class="time">'
-		clock = time.strftime(self.config["timeformat"], tm)
+		clock = safe_ftime(self.config["timeformat"], tm)
 		print >>self.file, '<h3>' + clock + '</h3>'
 		self.counter += 1
 
@@ -788,16 +807,19 @@ class AddFeedEditor:
 			outputfile.write("\n")
 		outputfile.write(self.feedline)
 
-def add_feed(filename, url, config):
+def add_feed(filename, url, rawdog, config):
 	"""Try to add a feed to the config file."""
-	feeds = feedfinder.getFeeds(url)
+	feeds = feedfinder.feeds(url)
 	if feeds == []:
 		print >>sys.stderr, "Cannot find any feeds in " + url
 	else:
 		feed = feeds[0]
-		print >>sys.stderr, "Adding feed " + feed
-		feedline = "feed %s %s\n" % (config["newfeedperiod"], feed)
-		edit_file(filename, AddFeedEditor(feedline).edit)
+		if feed in rawdog.feeds:
+			print >>sys.stderr, "Feed " + feed + " is already in the config file"
+		else:
+			print >>sys.stderr, "Adding feed " + feed
+			feedline = "feed %s %s\n" % (config["newfeedperiod"], feed)
+			edit_file(filename, AddFeedEditor(feedline).edit)
 
 class ChangeFeedEditor:
 	def __init__(self, oldurl, newurl):
@@ -810,6 +832,36 @@ class ChangeFeedEditor:
 				line = line.replace(self.oldurl, self.newurl, 1)
 			outputfile.write(line)
 
+class RemoveFeedEditor:
+	def __init__(self, url):
+		self.url = url
+	def edit(self, inputfile, outputfile):
+		while 1:
+			l = inputfile.readline()
+			if l == "":
+				break
+			ls = l.strip().split(None)
+			if len(ls) > 2 and ls[0] == "feed" and ls[2] == self.url:
+				while 1:
+					l = inputfile.readline()
+					if l == "":
+						break
+					elif l[0] == "#":
+						outputfile.write(l)
+					elif l[0] not in string.whitespace:
+						outputfile.write(l)
+						break
+			else:
+				outputfile.write(l)
+
+def remove_feed(filename, url, config):
+	"""Try to remove a feed from the config file."""
+	if url not in [f[0] for f in config["feedslist"]]:
+		print >>sys.stderr, "Feed " + url + " is not in the config file"
+	else:
+		print >>sys.stderr, "Removing feed " + url
+		edit_file(filename, RemoveFeedEditor(url).edit)
+
 class FeedFetcher:
 	"""Class that will handle fetching a set of feeds in parallel."""
 
@@ -1001,15 +1053,19 @@ class Rawdog(Persistable):
 			if rc:
 				seen_some_items[url] = 1
 
+		expiry_list = []
 		feedcounts = {}
 		for key, article in self.articles.items():
 			url = article.feed
 			feedcounts[url] = feedcounts.get(url, 0) + 1
+			expiry_list.append((article.added, article.sequence, key, article))
+		expiry_list.sort()
 
 		count = 0
-		for key, article in self.articles.items():
+		for date, seq, key, article in expiry_list:
 			url = article.feed
 			if (seen_some_items.has_key(url)
+			    and self.feeds.has_key(url)
 			    and article.can_expire(now, config)
 			    and feedcounts[url] > self.feeds[url].get_keepmin(config)):
 				plugins.call_hook("article_expired", self, config, article, now)
@@ -1339,6 +1395,7 @@ General options (use only once):
 -d|--dir DIR                 Use DIR instead of ~/.rawdog
 -v, --verbose                Print more detailed status information
 -N, --no-locking             Do not lock the state file
+-W, --no-lock-wait           Exit silently if state file is locked
 --help                       Display this help and exit
 
 Actions (performed in order given):
@@ -1351,6 +1408,7 @@ Actions (performed in order given):
 -T, --show-itemtemplate      Print the item template currently in use
 -a|--add URL                 Try to find a feed associated with URL and
                              add it to the config file
+-r|--remove URL              Remove feed URL from the config file
 
 Special actions (all other options are ignored if one of these is specified):
 --upgrade OLDDIR NEWDIR      Import feed state from rawdog 1.x directory
@@ -1364,7 +1422,7 @@ def main(argv):
 	locale.setlocale(locale.LC_ALL, "")
 
 	try:
-		(optlist, args) = getopt.getopt(argv, "ulwf:c:tTd:va:N", ["update", "list", "write", "update-feed=", "help", "config=", "show-template", "dir=", "show-itemtemplate", "verbose", "upgrade", "add=", "no-locking"])
+		(optlist, args) = getopt.getopt(argv, "ulwf:c:tTd:va:r:NW", ["update", "list", "write", "update-feed=", "help", "config=", "show-template", "dir=", "show-itemtemplate", "verbose", "upgrade", "add=", "remove=", "no-locking", "no-lock-wait"])
 	except getopt.GetoptError, s:
 		print s
 		usage()
@@ -1379,9 +1437,13 @@ def main(argv):
 		usage()
 		return 1
 
-	statedir = os.environ["HOME"] + "/.rawdog"
+	if "HOME" in os.environ:
+		statedir = os.environ["HOME"] + "/.rawdog"
+	else:
+		statedir = None
 	verbose = 0
 	locking = 1
+	no_lock_wait = 0
 	for o, a in optlist:
 		if o == "--help":
 			usage()
@@ -1392,6 +1454,11 @@ def main(argv):
 			verbose = 1
 		elif o in ("-N", "--no-locking"):
 			locking = 0
+		elif o in ("-W", "--no-lock-wait"):
+			no_lock_wait = 1
+	if statedir is None:
+		print "$HOME not set and state dir not explicitly specified; please use -d/--dir"
+		return 1
 
 	try:
 		os.chdir(statedir)
@@ -1399,6 +1466,8 @@ def main(argv):
 		print "No " + statedir + " directory"
 		return 1
 
+	sys.path.append(".")
+
 	config = Config()
 	try:
 		config.load("config")
@@ -1411,7 +1480,9 @@ def main(argv):
 
 	persister = Persister("state", Rawdog, locking)
 	try:
-		rawdog = persister.load()
+		rawdog = persister.load(no_block = no_lock_wait)
+		if rawdog is None:
+			return 0
 	except KeyboardInterrupt:
 		return 1
 	except:
@@ -1450,7 +1521,11 @@ def main(argv):
 		elif o in ("-T", "--show-itemtemplate"):
 			rawdog.show_itemtemplate(config)
 		elif o in ("-a", "--add"):
-			add_feed("config", a, config)
+			add_feed("config", a, rawdog, config)
+			config.reload()
+			rawdog.sync_from_config(config)
+		elif o in ("-r", "--remove"):
+			remove_feed("config", a, config)
 			config.reload()
 			rawdog.sync_from_config(config)
 
